/*
 * This macro allows you to use a 'C' function to handle an 8096 interrupt.
 *
 * The only parameter is the DECIMAL address of the interrupt vector.
 *
 * The macro MUST be placed IMMEDIATELY preceeding the definition of the
 * function which is to be the interrupt handler. Example:
 *
 * 		INTERRUPT(_SER_) serial_int_handler()
 * 		{
 *			/* ... Function code to handle serial interrupt ... */
 *		}
 *
 *
 * Unfortunately, the code generated by the compiler and in the runtime
 * library has to reference registers by address. This means that we cannot
 * simply "switch" the register bank during the interrupt, but have to
 * "push/pop" all of the registers instead.
 *
 * This file **REQUIRES** the extended pre-processor (MCP).
 * (CC96 ... -P, or "Preprocess" step enabled in DDSIDE)
 *
 * Copyright 1991-2005 Dave Dunfield
 * All rights reserved.
 */
#define	INTERRUPT(vec) asm {\
$SE:0						/* Insure output in code segment */\
_/**/vec	/* This label will be the address of the "prefix" stub */\
		ORG		vec			/* Position to interrupt vector */\
		DW		_/**/vec	/* Insert address of handler */\
		ORG		_/**/vec	/* Position back to original PC */\
		PUSHF				/* Save flags & disable */\
		PUSH	AX\
		PUSH	BX\
		PUSH	CX\
		PUSH	DX\
		PUSH	SI\
		PUSH	DI\
		SCALL	__/**/vec	/* Call the handler */\
		POP		DI\
		POP		SI\
		POP		DX\
		POP		CX\
		POP		BX\
		POP		AX\
		POPF				/* Restore flags */\
		RET					/* And return */\
__/**/vec	/* This label addresses the handler code */\
}

/*
 * Common 8096 vector addresses (must be DECIMAL)
 */
#define	TIMER	8192		/* Timer overflow 2000h */
#define	ADC		8194		/* ADC complete   2002h */
#define	HSID	8196		/* HSI data avail 2004h */
#define	HSOE	8198		/* HSO execution  2006h */
#define	HSIO	8200		/* HSI.O          2008h */
#define	STIMER	8202		/* Software timer 200Ah */
#define	SIO		8204		/* Serial I/O     200Ch */
#define	EXTI	8206		/* External INT   200Eh */
#define	TRAP    8208		/* Software INT   2010h */
