*#cpu 8096
*
* DDS MICRO-C Runtime Library for: 8096 Family
*
* This file contains the basic 8096 runtime library functions.
*
* Copyright 1991-2005 Dave Dunfield
* All rights reserved.
*
* Register definitions (Similar to 8086)
SP	EQU	$18		Stack pointer
BP	EQU	SP+2		Stack base pointer
AX	EQU	$1C		Accumulator
DX	EQU	AX+2		Accumulator extension
CX	EQU	AX+4		Runtime parameter
BX	EQU	AX+6		Temporary location
SI	EQU	AX+8		Index register
DI	EQU	AX+10		Stack addressing
?SPSTAT	EQU	DI+2		Serial port status temporary
* 8096 Internal register definitions
?IOC1	EQU	$16		I/O control 1
?IOPOR1	EQU	$0F		I/O port 1
?SPDATA	EQU	$07		Serial port DATA register
?SPCTRL	EQU	$11		Serial port CONTROL register
?SPBAUD	EQU	$000E		Serial port BAUDRATE register
*
* Startup code for Intel EV80C196KB Evaluation board
*
	ORG	$200C
	DW	?SERINT		Serial port interrupt vector
	ORG	$2080
	LD	SP,#$8000	Initial stack pointer
* Initialize the serial port
	LDB	?IOC1,#%00100000	Set P2.0 to TXD
	LDB	?SPBAUD,#$4D		12Mhz (196) = 9600
	LDB	?SPBAUD,#$80		Enable XCLK
	LDB	?SPCTRL,#%01001001	Enable Mode-1
	LDB	AX,?SPDATA		Clear TX port
	LDB	?SPSTAT,#%00100000	Fake TX set
	LDB	$08,#%01000000		Enable serial INTERRUPT
	EI
* Zero heap and invoke users "main" function
	ST	0,?heap		Zero the heap
	LCALL	main		Execute user function
* On exit, just sit and flash EV80C196KB's LED's
exit	DEC	AX		Decrement counter
	JNE	exit		Not expired, wait
	INCB	?IOPOR1		Advance LED value
	SJMP	exit		Keep looping
* Compare library routines
?eq	CMP	AX,CX		AX == CX
	JE	?true
	CLR	AX
	RET
?ne	CMP	AX,CX		AX != CX
	JNE	?true
	CLR	AX
	RET
?lt	CMP	AX,CX		AX < CX
	JLT	?true
	CLR	AX
	RET
?le	CMP	AX,CX		AX <= CX
	JLE	?true
	CLR	AX
	RET
?gt	CMP	AX,CX		AX > CX
	JGT	?true
	CLR	AX
	RET
?ge	CMP	AX,CX		AX >= CX
	JGE	?true
	CLR	AX
	RET
?ult	CMP	AX,CX		AX < CX (unsigned)
	JNC	?true
	CLR	AX
	RET
?ule	CMP	AX,CX		AX <= CX (unsigned)
	JNH	?true
	CLR	AX
	RET
?ugt	CMP	AX,CX		AX > CX (unsigned)
	JH	?true
	CLR	AX
	RET
?uge	CMP	AX,CX		AX >= CX (unsigned)
	JC	?true
?false	CLR	AX
	RET
* logically negate (AX)
?not	AND	AX,AX		Test for non-zero
	JNE	?false		Not zero, invert
?true	LD	AX,#1		Create a true value
	AND	AX,AX		Set condition code
nargs	RET
* Handle switch statement
?switch	LD	CX,[BX]		Get label value
	AND	CX,CX		End of table?
	JE	?swend		Special case
	CMP	AX,2[BX]	Does it match?
	JE	?idcall		Yes, execute
	ADD	BX,#4		Skip to next
	SJMP	?switch		Keep going
?swend	LD	CX,2[BX]	Get address
?idcall	BR	CX		Execute case code
*
* Interrupt handler for serial port...
* Simulates "normal" RX and TX ready bits by setting bits in our
* own status register when the Receive or Transmit interrupt occurs.
*
?SERINT	PUSHF			Save flags
	ORB	?SPSTAT,?SPCTRL	Save interrupt state
	POPF			Restore flags
	RET
$SE:1
*#map1 Segment 1/2 - Initialized variables
	ORG	(*+1)&-2
$SE:3
*#map3 Segment 3/4 - "register" variables
	ORG	?SPSTAT+2
