*
* Common routine to process a format specifier, and build an
* output string with appropriate substitutions.
*
* BX	- Pointer to next argument
* DI	- Pointer to output string
*
_format_ LD	SI,[BX]		Get format string
* Process next character from format string
?1	LDB	AX,[SI]+	Get character from string
	CMPB	AX,#'%'		Format specifier?
	JE	?2		Yes, handle it
* Non-format, copy to string
	STB	AX,[DI]+	Write to output string
	ANDB	AX,AX		End of string?
	JNE	?1		No, keep going
	RET
* Test for '-' (Left justify)
?2	LDB	AX,[SI]		Get next char
	LDB	AX+1,#' '	Assume space
	CMPB	AX,#'-'		Left justify?
	JNE	?3		No, its OK
	ORB	AX+1,#%10000000	Set LJ flag
	INC	SI		Skip '-'
	LDB	AX,[SI]		Get next char
* Test for leading '0' (zero fill)
?3	CMPB	AX,#'0'		Zero fill?
	JNE	?4		No, its OK
	ORB	AX+1,#'0'	Convert to ZF
?4	CLRB	CX		Zero initial width
* Evaluate field width
?5	LDB	AX,[SI]+	Get next char
	SUBB	AX,#'0'		Convert
	CMPB	AX,#10		In range?
	JC	?6		No, exit
	MULUB	CX,#10		Base 10
	ADDB	CX,AX		Include this digit
	SJMP	?5		Get next digit
* Interpret the format specifier
?6	LDB	CX+1,AX+1	Save flags
	DEC	BX		Backup
	DEC	BX		Backup
	LD	DX,[BX]		DX = next argument
	LDB	AX,-1[SI]	Get char back
	PUSH	SI		Save format
	PUSH	BX		Save argptr
* 'b' Binary number
	LDB	AX+1,#2		Base 2
	CMPB	AX,#'b'		Binary output?
	JE	?8		Yes, do it
* 'o' Octal number
	LDB	AX+1,#8		Base 8
	CMPB	AX,#'o'		Octal output?
	JE	?8		Yes, do it
* 'x' hexidecimal number
	LDB	AX+1,#16	Base 16
	CMPB	AX,#'x'		Hex output?
	JE	?8		Yes, do it
* 'u' unsigned number
	CMPB	AX,#'u'		Unsigned number?
	JE	?7		Yes, do it
* 'd' decimal number
	CMPB	AX,#'d'		Decimal number?
	JNE	?20		No, try next
	JBC	DX+1,7,?7	Not negative
	NEG	DX		Negate it
	LDB	AX,#'-'		Minus size
	STB	AX,[DI]+	Write to output
	ANDB	CX,CX		Field zero?
	JE	?7		Don't update
	DECB	CX		Reduce field width
?7	LDB	AX+1,#10	Base 10 output
* Output number in DX using base in AX+1
?8	LDBZE	SI,AX+1		Get number base
	CLRB	BX		Zero count
	PUSH	0		Terminator
* Process value & stack digits (in reverse order)
	LD	AX,DX		AX = value
?9	CLR	DX		Zero high
	DIVU	AX,SI		Do division
	CMPB	DX,#10		Are we over?
	JNC	?10		No, its OK
	ADDB	DX,#7		Convert ALPHAS
?10	ADDB	DX,#'0'		Convert to ASCII
	PUSH	DX		Save on stack
	INCB	BX		Advance count
	AND	AX,AX		More digits?
	JNE	?9		Yes, continue
* Unstack digits into output buffer
	SCALL	?40		Left fill if necessary
?11	POP	AX		Get char
	ANDB	AX,AX		End of line?
	JE	?12		Yes, quit
	STB	AX,[DI]+	Write to output
	SJMP	?11		Keep going
?12	JBC	CX+1,7,?13	No Right justify
	SCALL	?50		Right fill
?13	POP	BX		Restore ARGPTR
	POP	SI		Restore Format string
	SJMP	?1		And get next
* 's' string
?20	CMPB	AX,#'s'		String output?
	JNE	?30		No, try next
	PUSH	DX		Save for later
	LDB	BX,#-1		Initial count
?21	INCB	BX		Advance count
	LDB	AX,[DX]+	Test and advance
	ANDB	AX,AX		End of string?
	JNE	?21		Find end
	POP	DX		Restore pointer
	SCALL	?40		Left fill if necessary
?22	LDB	AX,[DX]+	Get char
	ANDB	AX,AX		At end?
	JE	?12		Yes, quit
	STB	AX,[DI]+	Write to output
	SJMP	?22		And proceed
*'c' Character
?30	CMPB	AX,#'c'		Character output?
	JE	?31		Yes, use char
	LD	DX,AX		Copy value to X
?31	LDB	BX,#1		Width=1
	SCALL	?40		Left fill if necessary
	STB	DX,[DI]+	Write char
	SJMP	?12		And proceed
*
* Pad the field assuming width in A
*
* DI=Output BX=?:width, CX=flag:fwidth
?40	JBC	CX+1,7,?50	No Left justify
?41	RET
?50	ANDB	CX,CX		Fixed field?
	JE	?41		No, exit
	LDB	AX+1,BX		Get actual width
	LDB	AX,CX+1		Get flags
	ANDB	AX,#%00111111	Get fill char
?52	CMPB	AX+1,CX		At field width?
	JC	?41		Yes, exit
	STB	AX,[DI]+	Write to output
	INCB	AX+1		Advance width
	SJMP	?52		And continue
