*
* Functions to perform basic arithmetic operations on arbitrarily
* large unsigned numbers. Supports: + - * / % <<1 >>1 == != < >
*
* To work on other than 32 bits, adjust the following constants:
?LSIZE	EQU	4		4 bytes (32 bits)
$DD:Longreg	4
*
* Shift a LONG value left: longshl(num1)
*
longshl	LD	DI,2[SP]	Get parameter
?bigshl	CLRC			No carry in
?bigslc	LDB	CX,#?LSIZE	Get byte count
?1	LDB	AX,[DI]		Get byte from num1
	ADDCB	AX,AX		Shift left with carry in
	STB	AX,[DI]+	Resave
	DJNZ	CX,?1		Shift them all
	LD	AX,#0		Assume zero
	ADDC	AX,AX		Return carry out
	RET
*
* Shift a LONG value right: longshr(num1)
*
longshr	LD	DI,2[SP]	Get parameter
?bigshr	CLR	AX		No carry in, zero AH
	LD	CX,#?LSIZE	Get byte count
	ADD	DI,CX		Advance to end
?2	SUB	DI,#1		Backup pointer
	LDB	AX,[DI]		Get byte from num1
	SHR	AX,#1		Shift it (rezero high)
	ADDCB	AX+1,AX+1	Ready carry for next
	STB	AX,[DI]		Resave
	DJNZ	CX,?2		Do them all
	LDB	AX,AX+1		Move to lower byte
	LDB	AX+1,#0		Zero AX
	RET
*
* Add two LONG numbers: longadd(num1, num2)
*
longadd	LD	DI,4[SP]	Get num1 pointer
	LD	SI,2[SP]	Get num2 pointer
?bigadd	LDB	CX,#?LSIZE	Get byte count
	CLRC			No carry in
?3	LDB	AX,[DI]		Get byte from num1
	ADDCB	AX,[SI]+	Add num2
	STB	AX,[DI]+	Resave
	DJNZ	CX,?3		Do them all
	LD	AX,#0		Zero AX
	ADDC	AX,AX		Return carry out
	RET
*
* Subtract two LONG numbers: longsub(num1, num2)
*
longsub	LD	DI,4[SP]	Get num1 pointer
	LD	SI,2[SP]	Get num2 pointer
?bigsub	LDB	CX,#?LSIZE	Get byte count
	SETC			No borrow in
?4	LDB	AX,[DI]		Get byte from num1
	SUBCB	AX,[SI]+	Subtract num2
	STB	AX,[DI]+	Resave
	DJNZ	CX,?4		Do them all
	LD	AX,#0		Zero AX
	ADDC	AX,AX		Return carry out
	RET
*
* Copy two LONG numbers: longcpy(num1, num2)
*
longcpy	LD	DI,4[SP]	Get num1 pointer
	LD	SI,2[SP]	Get num2 pointer
?bigcpy	LDB	CX,#?LSIZE	Get byte count
?5	LDB	AX,[SI]+	Get byte from num2
	STB	AX,[DI]+	Store in num1
	DJNZ	CX,?5		Do them all
	RET
*
* Set a long number: longset(num1, value)
*
longset	LD	DI,4[SP]	Get num1 pointer
	SCALL	?7		Zero it
	LD	DI,4[SP]	Get num1 pointer
	LD	AX,2[SP]	Get vlue
	ST	AX,[DI]		Save it
	RET
* Zero temporary location
?6	LD	DI,#Longreg	Point to temp reg
?7	LDB	CX,#?LSIZE	Get byte count
	CLR	AX		Zero AX
?8	STB	AX,[DI]+	Write zero
	DJNZ	CX,?8		Do them all
	RET
*
* Compare two long numbers: longcmp(num1, num2)
*
longcmp	LD	DI,4[SP]	Get pointer to num1
	LD	SI,2[SP]	Get pointer to num2
?bigcmp	LD	CX,#?LSIZE	Get byte count
	ADD	DI,CX		Adjust DI
	ADD	SI,CX		Adjust SI
?9	SUB	DI,#1		Backup num1
	SUB	SI,#1		Backup num2
	LDB	AX,[DI]		Get byte from num1
	CMPB	AX,[SI]		Compare with num2
	JH	?10		Its higher
	JNE	?11		Its lower
	DJNZ	CX,?9		Do them all
	CLR	AX		Values are equal
	RET
?10	LD	AX,#1		Indicate num1 > num2
	RET
?11	LD	AX,#-1		Indicate num1 < num2
	RET
*
* Test a long number for zero: longtst(num1)
longtst	LD	DI,2[SP]	Get pointer to num1
?bigtst	LDB	CX,#?LSIZE	Get byte count
	CLR	AX		Zero AX
?12	ORB	AX,[DI]+	Test it
	JNE	?13		Non zero-
	DJNZ	CX,?12		Do them all
?13	RET
*
* Multiply two LONG numbers: longmul(num1, num2)
*
longmul	SCALL	?6		Zero temp register
	SUB	SP,#?LSIZE	Reserve space
	LD	DI,SP		Point to temp
	LD	SI,?LSIZE+2[SP]	Point to num2
	SCALL	?bigcpy		Copy it
?14	LD	DI,?LSIZE+4[SP]	Get num1 ptr
	SCALL	?bigshr		Shift it
	AND	AX,AX		Test for carry out
	JE	?15		No, don't add
	LD	DI,#Longreg	Point to temp reg
	LD	SI,SP		Point to num3
	SCALL	?bigadd		Do the addition
?15	LD	DI,?LSIZE+4[SP]	Get num1 ptr
	SCALL	?bigtst		Is it zero?
	JE	?16		Yes, quit
	LD	DI,SP		Point to num3
	SCALL	?bigshl		Shift it
	LD	DI,SP		Point to num3
	SCALL	?bigtst		Is it zero
	JNE	?14		Do it all
?16	ADD	SP,#?LSIZE	Fix stack
	LD	DI,4[SP]	Get num1 ptr
	LD	SI,#Longreg	Get temp reg
	SJMP	?bigcpy		Copy it
*
* Divide two LONG numbers: longdiv(num1, num2)
*
longdiv	SCALL	?6		Zero temp storage
	LDB	DX,#(?LSIZE*8)+1 Get bit count
?17	CLRC			No carry in
?18	LD	DI,4[SP]	Get num1 pointer
	SCALL	?bigslc		Shift in carry
	DECB	DX		Reduce count
	JE	?13		End, exit
	LD	DI,#Longreg	Point to temp reg
	SHRB	AX,#1		Set cy
	SCALL	?bigslc		Shift it
	LD	DI,#Longreg	Point to temp trg
	LD	SI,2[SP]	Get num2 pointer
	SCALL	?bigcmp		Compare them
	ANDB	AX+1,AX+1	Less than?
	JNE	?17		Yes, carry in 0
	LD	DI,#Longreg	Point to temp reg
	LD	SI,2[SP]	Point to num2
	SCALL	?bigsub		Do subtract
	SETC			Set carry in
	SJMP	?18		And proceed
