*
* Serial I/O functions for 8096 family processor
*
* Write a string to the serial port
putstr	LD	SI,2[SP]	Get string
?putstr	LDB	AX,[SI]+	Get char
	ANDB	AX,AX		End of string?
	JE	?1		Yes, exit
	SCALL	?putch		Write it
	SJMP	?putstr		Do it all
*
* Write char with NEWLINE<>CR translation
*
putch	LDB	AX,2[SP]	Get char to write
?putch	SCALL	?putchr		Write value
	CMPB	AX,#$0A		Newline?
	JNE	?1		No, skip it
	LDB	AX,#$0D		Add carriage return
	SJMP	?putchr		Write it out
*
* Write char with no translations
*
putchr	LDB	AX,2[SP]	Get char to write
?putchr	JBC	?SPSTAT,5,*	Wait till empty
	ANDB	?SPSTAT,#%11011111 Clear flag
	LDB	?SPDATA,AX	Write char to uart
?1	RET
*
* Test for character (No translation)
*
chkchr	LD	AX,#-1		Assume no char
	JBC	?SPSTAT,6,?1	No character
*
* Receive char with no translations
*
getchr	JBC	?SPSTAT,6,*	Wait for RX bit
	ANDB	?SPSTAT,#%10111111 Clear flag
	LDBZE	AX,?SPDATA	Read char & zero high
	RET
*
* Test for character (with CR<>NEWLINE translation)
*
chkch	CLR	AX		Assume NO char
	JBC	?SPSTAT,6,?1	No character
*
* Receive char with CR<>NEWLINE translation
*
getch	SCALL	getchr		Get the character
	CMPB	AX,#$0D		Carriage return
	JNE	?1		No, its OK
	LDB	AX,#$0A		Convert to NEWLINE
	RET
*
* Recieve a string: getstr(buffer, size)
*
getstr	LD	SI,4[SP]	Get buffer address
	CLR	CX		Zero CX
?2	SCALL	getch		Get character
	CMPB	AX,#$08		Backspace?
	JE	?3		Yes, handle it
	CMPB	AX,#$7F		Delete?
	JE	?3		Yes, handle it
	CMPB	AX,#$0A		End of line?
	JE	?4		Yes, handle it
	CMP	CX,2[SP]	Are we at end?
	JC	?2		Yes, don't accept
	STB	AX,[SI]+	Write it
	INC	CX		Advance length
	SCALL	?putch		Write it out
	SJMP	?2		And proceed
* Delete character from buffer
?3	CMP	CX,0		At beginning?
	JE	?2		Yes, ignore
	LDB	AX,#$08		Backup
	SCALL	?putch		Output
	LDB	AX,#' '		Space over
	SCALL	?putch		Output
	LDB	AX,#$08		Backup
	SCALL	?putch		Output
	DEC	SI		Backup pointer
	DEC	CX		Reduce length
	SJMP	?2		And proceed
* Newline, terminate entry
?4	SCALL	?putch		Echo it
	STB	0,[SI]		Zero terminate
	LD	AX,CX		Get length
	RET
